#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <time.h>

// Remember, operator mutation is half of this. 
#define MUTATE_PROB 0.05
#define K 3

typedef uint32_t state_t;

typedef struct {
	uint32_t *table;  // size 1<<K
	uint32_t *input;  // size K
} node_t;
// Gj�r om til byte, ellers blir det her stort.

typedef enum {VISITED, NOT_VISITED} Visited;

node_t *network_create (int num_nodes) {
	node_t *network = malloc(num_nodes*sizeof(node_t));
	for (int i = 0; i < num_nodes; i++) {
		network[i].table = malloc((1<<K)*sizeof(state_t));
		network[i].input = malloc(K*sizeof(state_t));
		for (int j = 0; j < (1<<K); j++) {
		network[i].table[j] = rand() % 2;
		}
		for (int k = 0; k < K; k++){
		network[i].input[k] = rand() % num_nodes;
		}
	}
	return network;
}

void network_destroy (node_t *network, int num_nodes) {
	for (int i = 0; i < num_nodes; i++) {
		free(network[i].table);
		free(network[i].input);
	}
	free(network);
}

//New state calculation + pointer conversion.
state_t ptr_to_state (state_t *state, int num_nodes) {
	int ret = 0;
	for (int i = 0; i < num_nodes; i++) {
		ret |= (state[i])<<i;
	}
	return ret;
}

state_t *state_to_ptr (state_t state, int num_nodes) {
	state_t *ret_ptr = malloc(sizeof(state_t) * num_nodes);
	for (int i = 0; i < num_nodes; i++) {
		ret_ptr[i] = (state>>i)&1;
	}
	return ret_ptr;
}

uint32_t bin_to_dec(uint32_t len, uint32_t *bins) {
	uint32_t dec = 0;
	for (int i = 0; i < len; i++) {
		dec = (dec<<1) | bins[i];
	}
	free(bins);
	return dec;
}

state_t calc_new_state(node_t *network, state_t *prev_state, int node_index) {
	uint32_t *table = network[node_index].table;
	state_t *input_nodes = network[node_index].input;
	uint32_t *bins = malloc(K*sizeof(uint32_t));
	for (int i = 0; i < K; i++) {
		bins[i] = prev_state[input_nodes[i]]; // ?????
	}
	return table[bin_to_dec(K, bins)];
}

state_t next_state (node_t *network, state_t previous_state, int num_nodes) {
	state_t *next_state = malloc(sizeof(state_t) * num_nodes);
	state_t *prev_state = state_to_ptr(previous_state, num_nodes);
	for (int i = 0; i < num_nodes; i++) {
		next_state[i] = calc_new_state(network, prev_state, i);
		if (next_state[i] == -1) {
			printf("Illegal node operator. Terminating.\n");
			exit(1);
		}
	}
	state_t result = ptr_to_state(next_state, num_nodes);
	free(prev_state);
	free(next_state);
	return result;
}

// Generating random states.
float randomFloat() {
      float r = (float)rand()/(float)RAND_MAX;
      return r;
}

state_t *generate_random_states(int num_nodes) {
	int num_states = 1<<num_nodes;
	state_t * result = malloc(sizeof(state_t)*num_states);	
	for (int i = 0; i < num_states; i++) {
		result[i] = rand() % 2;
	}
	return result;
}

void network_mutate(node_t *network, int num_nodes){
	for (int i = 0; i < num_nodes; i++) {
		if (MUTATE_PROB/2 > randomFloat()) {
			int mutate_index = rand() % (1<<K);
			int val = network[i].table[mutate_index];
			network[i].table[mutate_index] = !val;
			// Guarantees Hamming distance of 1 to mutated truth table.
		}
		for (int j = 0; j < K; j++) {
			if (MUTATE_PROB > randomFloat()) {
				network[i].input[j] = rand() % num_nodes;
			}
		}
	}
}

void state_history_save(state_t *state_history, Visited *visited, int iteration) {
	for (int i = 0; i < iteration; i++) {
		visited[state_history[i]] = VISITED;
	}
	free(state_history);
}

int network_fitness(node_t *network, int num_nodes, int sample_divisor) {
	int num_states = 1<<num_nodes;
	int max_ticks = num_states+1;
	int max_attractor_len = 0;
	state_t * init_states = generate_random_states(num_nodes);
	Visited *visited = malloc(sizeof(Visited)*num_states);
	for (int i = 0; i < num_states; i++) {
		visited[i] = NOT_VISITED;
	}

	for (int i = 0; i < num_states/sample_divisor; i++) {
		state_t start_state = init_states[i];
		
		if (visited[start_state] == VISITED) {
			goto endcycle;	
		}
		state_t *state_history = malloc(sizeof(state_t)*num_states);
		state_history[0] = start_state;
		state_t new_state;
		//Check whether start_state is visited.
		int j = 0;
		while (j < max_ticks) {
			j++;
			new_state = next_state(network, state_history[j-1], num_nodes);
			if (visited[new_state] == VISITED) {
				state_history_save(state_history, visited, j);
				break;
			}
			if (memcmp(&state_history[j-1], &new_state, sizeof(state_t)) == 0) {
				if (max_attractor_len < 1) {
					max_attractor_len = 1;
				}
				state_history_save(state_history, visited, j);
				break;
			}
			else {
				for (int k = 0; k < j; k++) { // Cycle through history, starting at 0 because it breaks the loop on first detected cycle anyway.
					if (memcmp(&state_history[k], &new_state, sizeof(state_t)) == 0) {
						if (max_attractor_len < (j-k)) {
						max_attractor_len = (j-k);
						}
						state_history_save(state_history, visited, j);
						goto endcycle; // Because break doesn't break out of the while loop.
					}
				}
			}
			state_history[j] = new_state;
		}
		endcycle: ; //Labels are associated with statements and all that.
	}
	free(init_states);
	free(visited);
	return max_attractor_len;
}

node_t * network_reproduce(node_t *parent_one, node_t *parent_two, int num_nodes) {
	node_t * child = malloc(num_nodes*sizeof(node_t));
	for (int i = 0; i < num_nodes; i++) {
		child[i].table = malloc((1<<K)*sizeof(state_t));
		child[i].input = malloc(K*sizeof(state_t));
	}
	int crossover_point = rand() % num_nodes;
	for (int i = 0; i < crossover_point; i++) {
		memcpy(child[i].table, parent_one[i].table, (1<<K)*sizeof(state_t));
		memcpy(child[i].input, parent_one[i].input, K*sizeof(state_t));
	}
	for (int j = crossover_point; j < num_nodes; j++) {
		memcpy(child[j].table, parent_two[j].table, (1<<K)*sizeof(state_t));
		memcpy(child[j].input, parent_two[j].input, K*sizeof(state_t));
	}
	network_mutate(child, num_nodes);
	return child;
}

void nextBest(int *arr, int n, int *max, int *maxIndex) {
  int cand = -1;
  int candIndex = -1;
  for (int i = 0; i < n; i++) {
    // max cand < arr[i] < cur max
    if ((cand < arr[i]) && (arr[i] < *max)) {
      cand = arr[i];
      candIndex = i;
      continue;
    }
    // arr[i] == max, and maxIndex < i
    if ((cand < arr[i]) && (arr[i] == *max) && (*maxIndex < i)) {
      cand = arr[i];
      candIndex = i;
      continue;
    }
  }
  *max = cand;
  *maxIndex = candIndex;
}

void array_print(node_t *network, int num_nodes, FILE * file) {
	fprintf(file, "{");
	for (int i = 0; i < num_nodes-1; i++) {
		for (int j = 0; j < 1<<K; j++) {
			fprintf(file, "%d", network[i].table[j]);
		}
		fprintf(file, ", ");
	}
	for (int j = 0; j < 1<<K; j++) {
			fprintf(file, "%d", network[num_nodes-1].table[j]);
	}
	fprintf(file,"} \n");
	for (int j = 0; j < K; j++) {
		fprintf(file, "{");
		for (int i = 0; i < num_nodes-1; i++) {
			fprintf(file,"%d, ", network[i].input[j]);
		}
		fprintf(file,"%d} \n", network[num_nodes-1].input[j]);
	}
}

int main(int argc, char *argv[]) {
	if (argc < 7) {
		printf("Too few arguments. Arguments needed: \n Stat file name, graph file name, # nodes, # generations, # population, # elites\n");
		exit(1);
	}
	unsigned int seed; 
	FILE* urandom = fopen("/dev/urandom", "r");
	fread(&seed, sizeof(int), 1, urandom);
	fclose(urandom); 
	srand(seed);
	FILE *stats;
	FILE *graph;
	char *statsfile = argv[1];
	char *graphfile = argv[2];
	int num_nodes = atoi(argv[3]);
	int total_gens = atoi(argv[4]);
	int pop_count = atoi(argv[5]);
	int elite_num = atoi(argv[6]);
	
	stats = fopen (statsfile, "w");
	graph = fopen (graphfile, "w");
	
	int total_fitness = 0;
	int sample_divisor = 1; // For reducing the number of states tested.
	int fitness_val = 0;
	int pi1 = 0; // Parent indexes.
	int pi2 = 0;
	int rand1 = 0; // Roulette variables.
	int rand2 = 0;
	int max_fitness;
	int min_fitness;
	double mean_fitness;
	double sd;
	double temp;
	int max; // For elitism.
	int maxIndex;

	struct timespec time_start, time_stop;
	long long nanoseconds;

	clock_gettime(CLOCK_MONOTONIC, &time_start);
	
	node_t ** population_array = malloc(sizeof(node_t*)*pop_count);
	int * fitness_array = malloc(sizeof(int)*pop_count);	
	for (int i = 0; i < pop_count; i++) {
			population_array[i] = network_create(num_nodes);
	}
		
	//Actual evolution comes here:
	for (int i = 0; i < total_gens; i++) {
		node_t ** child_array = malloc(sizeof(node_t*)*pop_count);
		
		// Calculating MAX, MIN, MEAN, SD.
		max_fitness = 0;
		min_fitness = 1<<num_nodes;
		mean_fitness = 0.0;
		sd = 0.0;
		temp = 0.0;
		total_fitness = 0;
		
		for (int j = 0; j < pop_count; j++) {
			fitness_val = network_fitness(population_array[j], num_nodes, sample_divisor);
			total_fitness += fitness_val;
			if (fitness_val < min_fitness) min_fitness = fitness_val;
			if (fitness_val > max_fitness) max_fitness = fitness_val;
			fitness_array[j] = fitness_val;
		}
		
		mean_fitness = (double)total_fitness / (double)pop_count;
		for (int o = 0; o < pop_count; o++) {
			temp += pow((fitness_array[o]-mean_fitness), 2.0);
		}
		sd = sqrt(temp/((double)pop_count));
		//Format: GEN MAX MIN MEAN SD
		fprintf(stats, "%d %d %d %.5f %.5f \n", i, max_fitness, min_fitness, mean_fitness, sd);
		
		max = total_fitness;
		maxIndex = -1;
		// Network reproduction happens here. Roulette wheel strategy + elitism
		for (int k = 0; k < elite_num; k++) {
			nextBest(fitness_array, pop_count, &max, &maxIndex);
			node_t * elite = malloc(num_nodes*sizeof(node_t));
			for(int i = 0; i < num_nodes; i++) {
				elite[i].table = malloc((1<<K)*sizeof(state_t));
				elite[i].input = malloc(K*sizeof(state_t));
				memcpy(elite[i].table, population_array[maxIndex][i].table, (1<<K)*sizeof(state_t));
				memcpy(elite[i].input, population_array[maxIndex][i].input, K*sizeof(state_t));
			}
			child_array[k] = elite;
		}
		
		for (int k = elite_num; k < pop_count; k++) {
			rand1 = rand() % total_fitness;
			rand2 = rand() % total_fitness;
			for (int m = 0; m < pop_count; m++) {
				if (fitness_array[m] < rand1) {
					pi1 = m;
					break;
				}
				else {
					rand1 -= fitness_array[m];
				}
			}
			for (int n = 0; n < pop_count; n++) {
				if (fitness_array[n] < rand2) {
					pi2 = n;
					break;
				}
				else {
					rand2 -= fitness_array[n];
				}
			}
			child_array[k] = network_reproduce(population_array[pi1], population_array[pi2], num_nodes);	
		}
		
		for(int i = 0; i < pop_count; i++) {
			network_destroy(population_array[i], num_nodes);
		}
		free(population_array);
		population_array = child_array;
	}
	
	// LAST GENERATION
	max_fitness = 0;
	min_fitness = 1<<num_nodes;
	mean_fitness = 0.0;
	sd = 0.0;
	temp = 0.0;
	total_fitness = 0;
	
	for (int j = 0; j < pop_count; j++) {
		fitness_val = network_fitness(population_array[j], num_nodes, sample_divisor);
		total_fitness += fitness_val;
		if (fitness_val < min_fitness) min_fitness = fitness_val;
		if (fitness_val > max_fitness) max_fitness = fitness_val;
		fitness_array[j] = fitness_val;
	}
	
	mean_fitness = (double)total_fitness / (double)pop_count;
	for (int o = 0; o < pop_count; o++) {
		temp += pow((fitness_array[o]-mean_fitness), 2.0);
	}
	sd = sqrt(temp/((double)pop_count));
	//Format: GEN MAX MIN MEAN SD
	fprintf(stats, "%d %d %d %.5f %.5f \n", total_gens, max_fitness, min_fitness, mean_fitness, sd);
	
	// Printing the best network(s) after M generations!
	for (int i = 0; i < pop_count; i++) {
		if (fitness_array[i] == max_fitness) {
			array_print(population_array[i], num_nodes, graph);
			break;
		}
	}
	fclose(stats);
	fclose(graph);
	for(int i = 0; i < pop_count; i++) {
		free(population_array[i]);
	}
	free(population_array);
	free(fitness_array);
	clock_gettime(CLOCK_MONOTONIC, &time_stop);
	nanoseconds = (time_stop.tv_sec - time_start.tv_sec) * 1000000000LL;
	nanoseconds += (time_stop.tv_nsec - time_start.tv_nsec);
	printf("Time: %lld \n", nanoseconds);
	return 0;
}
