#!/usr/bin/env bash

E_BADARGS=85

if [ "$#" -ne "5" ]; then
    echo "Usage: ./`basename $0` instrfile tablefile out ncount k"
    echo "Creates a pdf plot out of the aggregated data files."
    exit $E_BADARGS
fi

file1="$1"
file2="$2"
base="${3%%.*}"
ncount="$4"
k="$5"


# 1      2      3        4        5       6        7
# gen min-max 25-max median-max 75-max max-max mean-max
# 1      8      9       10        11     12       13
# gen min-min 25-min median-min 75-min max-min mean-min
# 1      14     15      16        17     18       19
# gen min-avg 25-avg median-avg 75-avg max-avg mean-avg
# 1      20     21      22        23     24       25
# gen min-std 25-std median-std 75-std max-std mean-std

gnuplot <<EOF
set terminal pdf mono font 'Bitstream Charter'
set termoption dash

set output "$base.pdf"

set title 'Aggregated fitness for K=$k, N=$ncount'
set key inside left top
#set key inside right bottom box linetype -1 linewidth 1.000 width 6

set xtic auto
set ytic auto

set xlabel 'Generation'
set ylabel 'Fitness'

plot "$file1" u 1:7 w lines lt 1 lc rgb 'blue' t "Max (Instruction based)",\
     '' u 1:19 w lines lt 2 lc rgb 'blue' t "Mean (Instruction based)",\
     "$file2" u 1:7 w lines lt 1 lc rgb 'red' t "Max (Table based)",\
     '' u 1:19 w lines lt 2 lc rgb 'red' t "Mean (Table based)"

# set terminal epslatex
# set output "$base.tex"
# replot
EOF

echo "$base.pdf generated."
