#!/usr/bin/env bash

set -euo pipefail
IFS=$'\n\t'

## remove "Time: " in files
find . -type f -iname 'time-*.txt' -exec sed -i 's/Time: //g' {} \;

## for all folders
for dir in */; do
    cd "$dir"
    echo "for $dir"
    if [ -f "time-aggregate.txt" ]; then
        echo 'removing existing time-aggregate.txt file'
        rm "time-aggregate.txt"
    fi
    for n in $(seq 10 2 18); do
        if [ -f  "time-$n.txt" ]; then
            ../time-aggregate.py "time-$n.txt" "$n" >> "time-aggregate.txt"
        fi
    done
    # PLOTTI-PLOTTI-PLOT
    if [ -f "time-aggregate.txt" ]; then
        gnuplot <<EOF
set terminal pdf mono font 'Bitstream Charter'
set termoption dash

set boxwidth 1.0

set output "$PWD/plotti-plot.pdf"
input="$PWD/time-aggregate.txt"
set title 'Box-and-whisker plot for runtime of $(echo "$dir" | sed -e 's/-/ /g' -e 's+/++g')'
set key inside left top box linetype -1 linewidth 1.000

set xrange [8:20]
set ytic auto
set logscale y

set xlabel 'Nodes'
set ylabel 'Milliseconds'

plot input u 1:(\$3/1000000):(\$2/1000000):(\$6/1000000):(\$5/1000000)\
       with candlesticks whiskerbars lt 1 lc rgb 'blue' t "Runtime",\
     '' u 1:(\$4/1000000):(\$4/1000000):(\$4/1000000):(\$4/1000000) \
      with candlesticks lt -1 notitle
EOF
    fi
    cd ..
done
