#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
// Remember, operator mutation is half of this. 


typedef uint32_t state_t;

typedef enum {				// 00 01 10 11
	FALSE = 0, 				// 0  0  0  0
	AND = 1,				// 0  0  0  1
	MAT_NONIMPL = 2,		// 0  0  1  0
	A_PROJ = 3,				// 0  0  1  1
	CONV_NONIMPL = 4,		// 0  1  0  0
	B_PROJ = 5, 			// 0  1  0  1
	XOR = 6,				// 0  1  1  0	
	OR = 7,		 			// 0  1  1  1	 
	NOR = 8,	 			// 1  0  0  0
	XNOR = 9,				// 1  0  0  1
	NOT_B = 10,				// 1  0  1  0
	CONV_IMPL = 11,			// 1  0  1  1
	NOT_A = 12,				// 1  1  0  0
	MAT_IMPL = 13,			// 1  1  0  1
	NAND = 14,				// 1  1  1  0
	TRUE = 15				// 1  1  1  1
} gate_t;

int result[256] = {0};

state_t eval(gate_t operator, state_t a, state_t b) {
	switch (operator) {
		case FALSE:
			return 0;
		case AND:
			return a & b;
		case MAT_NONIMPL:
			if (a == 1 && b == 0) return 1;
			return 0;
		case A_PROJ:
			return a;
		case CONV_NONIMPL:
			if (a == 0 && b == 1) return 1;
			return 0;
		case B_PROJ:
			return b;
		case XOR:
			return a ^ b;
		case OR:
			return a | b;
		case NOR:
			return !(a | b);
		case XNOR:
			return !(a ^ b);
		case NOT_B:
			return !b;
		case CONV_IMPL:
			if (a == 0 && b == 1) return 0;
			return 1;
		case NOT_A:
			return !a;
		case MAT_IMPL:
			if (a == 1 && b == 0) return 0;
			return 1;
		case NAND:
			return !(a & b);
		case TRUE:
			return 1;
	}
	return -1; // Just in case.
}

void struct_perm (gate_t opA, gate_t opB, state_t shift_a, state_t shift_b, state_t shift_c) {
	state_t a = 0, b = 0, c = 0;
	int value = 0;
	for(int i = 0; i < 8; i++) {
		a = (i>>shift_a) & 1;
		b = (i>>shift_b) & 1;
		c = (i>>shift_c) & 1;
		value |= (eval(opB, eval(opA, a, b), c))<<i;
	}
	result[value]++;
	value = 0;
	for(int i = 0; i < 8; i++) {
		a = (i>>shift_a) & 1;
		b = (i>>shift_b) & 1;
		c = (i>>shift_c) & 1;
		value |= (eval(opB, a, eval(opA, b, c)))<<i;
	}
	result[value]++;
	value = 0;
	for(int i = 0; i < 8; i++) {
		a = (i>>shift_a) & 1;
		b = (i>>shift_b) & 1;
		c = (i>>shift_c) & 1;
		value |= (eval(opA, eval(opB, a, b), c))<<i;
	}
	result[value]++;
	value = 0;
	for(int i = 0; i < 8; i++) {
		a = (i>>shift_a) & 1;
		b = (i>>shift_b) & 1;
		c = (i>>shift_c) & 1;
		value |= (eval(opA, a, eval(opB, b, c)))<<i;
	}
	result[value]++;
}

void input_perm(gate_t opA, gate_t opB) {
	struct_perm(opA, opB, 0, 1, 2);
	struct_perm(opA, opB, 0, 2, 1);
	struct_perm(opA, opB, 1, 0, 2);
	struct_perm(opA, opB, 1, 2, 0);
	struct_perm(opA, opB, 2, 0, 1);
	struct_perm(opA, opB, 2, 1, 0);
}

void result_print() {
	int count = 0;
	int num_zeros = 0;
	for (int i = 0; i < 256; i++){
		if (result[i] == 0) {
			num_zeros++;
		}
		count += result[i];
		printf("%d: %d \n", i, result[i]);
	}
	printf("number of zero entries: %d \n", num_zeros);
	printf("total: %d \n", count); 
}

int main() {
	for (int a = 0; a < 16; a++) {
		for (int b = 0; b < 16; b++) {
			input_perm((gate_t) a, (gate_t) b);
		}
	}
	result_print();
	return 0;
}
